@@ -11,6 +11,7 @@ import hashlib
 import os
 import shutil
 import ftplib
+from tornado.httputil import parse_response_start_line, HTTPInputError
 
 # Import salt libs
 from salt.exceptions import (
@@ -64,6 +65,19 @@ class Client(object):
         self.opts = opts
         self.serial = salt.payload.Serial(self.opts)
 
+    # Add __setstate__ and __getstate__ so that the object may be
+    # deep copied. It normally can't be deep copied because its
+    # constructor requires an 'opts' parameter.
+    # The TCP transport needs to be able to deep copy this class
+    # due to 'salt.utils.context.ContextDict.clone'.
+    def __setstate__(self, state):
+        # This will polymorphically call __init__
+        # in the derived class.
+        self.__init__(state['opts'])
+
+    def __getstate__(self):
+        return {'opts': self.opts}
+
     def _check_proto(self, path):
         '''
         Make sure that this path is intended for the salt master and trim it
@@ -92,21 +106,26 @@ class Client(object):
         return filelist
 
     @contextlib.contextmanager
-    def _cache_loc(self, path, saltenv='base', env=None):
+    def _cache_loc(self, path, saltenv='base', env=None, cachedir=None):
         '''
         Return the local location to cache the file, cache dirs will be made
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
 
-        dest = salt.utils.path_join(self.opts['cachedir'],
+        if cachedir is None:
+            cachedir = self.opts['cachedir']
+        elif not os.path.isabs(cachedir):
+            cachedir = os.path.join(self.opts['cachedir'], cachedir)
+
+        dest = salt.utils.path_join(cachedir,
                                     'files',
                                     saltenv,
                                     path)
@@ -127,7 +146,8 @@ class Client(object):
                  makedirs=False,
                  saltenv='base',
                  gzip=None,
-                 env=None):
+                 env=None,
+                 cachedir=None):
         '''
         Copies a file from the local files or master depending on
         implementation
@@ -140,34 +160,34 @@ class Client(object):
         '''
         raise NotImplementedError
 
-    def cache_file(self, path, saltenv='base', env=None):
+    def cache_file(self, path, saltenv='base', env=None, cachedir=None):
         '''
         Pull a file down from the file server and store it in the minion
         file cache
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
 
-        return self.get_url(path, '', True, saltenv)
+        return self.get_url(path, '', True, saltenv, cachedir=cachedir)
 
-    def cache_files(self, paths, saltenv='base', env=None):
+    def cache_files(self, paths, saltenv='base', env=None, cachedir=None):
         '''
         Download a list of files stored on the master and put them in the
         minion file cache
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -176,39 +196,42 @@ class Client(object):
         if isinstance(paths, str):
             paths = paths.split(',')
         for path in paths:
-            ret.append(self.cache_file(path, saltenv))
+            ret.append(self.cache_file(path, saltenv, cachedir=cachedir))
         return ret
 
-    def cache_master(self, saltenv='base', env=None):
+    def cache_master(self, saltenv='base', env=None, cachedir=None):
         '''
         Download and cache all files on a master in a specified environment
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
 
         ret = []
         for path in self.file_list(saltenv):
-            ret.append(self.cache_file(salt.utils.url.create(path), saltenv))
+            ret.append(
+                self.cache_file(
+                    salt.utils.url.create(path), saltenv, cachedir=cachedir)
+            )
         return ret
 
     def cache_dir(self, path, saltenv='base', include_empty=False,
-                  include_pat=None, exclude_pat=None, env=None):
+                  include_pat=None, exclude_pat=None, env=None, cachedir=None):
         '''
         Download all of the files in a subdir of the master
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -223,17 +246,19 @@ class Client(object):
             path = path + '/'
 
         log.info(
-            'Caching directory {0!r} for environment {1!r}'.format(
+            'Caching directory \'{0}\' for environment \'{1}\''.format(
                 path, saltenv
             )
         )
         # go through the list of all files finding ones that are in
         # the target directory and caching them
         for fn_ in self.file_list(saltenv):
+            fn_ = sdecode(fn_)
             if fn_.strip() and fn_.startswith(path):
                 if salt.utils.check_include_exclude(
                         fn_, include_pat, exclude_pat):
-                    fn_ = self.cache_file(salt.utils.url.create(fn_), saltenv)
+                    fn_ = self.cache_file(
+                        salt.utils.url.create(fn_), saltenv, cachedir=cachedir)
                     if fn_:
                         ret.append(fn_)
 
@@ -247,12 +272,14 @@ class Client(object):
             #     prefix = ''
             # else:
             #     prefix = separated[0]
-            dest = salt.utils.path_join(
-                self.opts['cachedir'],
-                'files',
-                saltenv
-            )
+            if cachedir is None:
+                cachedir = self.opts['cachedir']
+            elif not os.path.isabs(cachedir):
+                cachedir = os.path.join(self.opts['cachdir'], cachedir)
+
+            dest = salt.utils.path_join(cachedir, 'files', saltenv)
             for fn_ in self.file_list_emptydirs(saltenv):
+                fn_ = sdecode(fn_)
                 if fn_.startswith(path):
                     minion_dir = '{0}/{1}'.format(dest, fn_)
                     if not os.path.isdir(minion_dir):
@@ -280,10 +307,10 @@ class Client(object):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -301,10 +328,10 @@ class Client(object):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -317,10 +344,10 @@ class Client(object):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -333,27 +360,27 @@ class Client(object):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
 
         return {}
 
-    def is_cached(self, path, saltenv='base', env=None):
+    def is_cached(self, path, saltenv='base', env=None, cachedir=None):
         '''
         Returns the full path to a file if it is cached locally on the minion
         otherwise returns a blank string
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -370,12 +397,14 @@ class Client(object):
             self.opts['cachedir'], 'localfiles', path.lstrip('|/'))
         filesdest = os.path.join(
             self.opts['cachedir'], 'files', saltenv, path.lstrip('|/'))
-        extrndest = self._extrn_path(path, saltenv)
+        extrndest = self._extrn_path(path, saltenv, cachedir=cachedir)
 
         if os.path.exists(filesdest):
             return salt.utils.url.escape(filesdest) if escaped else filesdest
         elif os.path.exists(localsfilesdest):
-            return salt.utils.url.escape(localsfilesdest) if escaped else localsfilesdest
+            return salt.utils.url.escape(localsfilesdest) \
+                if escaped \
+                else localsfilesdest
         elif os.path.exists(extrndest):
             return extrndest
 
@@ -393,7 +422,7 @@ class Client(object):
         if limit_traversal:
             if saltenv not in self.opts['file_roots']:
                 log.warning(
-                    'During an attempt to list states for saltenv {0!r}, '
+                    'During an attempt to list states for saltenv \'{0}\', '
                     'the environment could not be found in the configured '
                     'file roots'.format(saltenv)
                 )
@@ -433,7 +462,7 @@ class Client(object):
                         states.append(path.replace('/', '.')[:-4])
         return states
 
-    def get_state(self, sls, saltenv):
+    def get_state(self, sls, saltenv, cachedir=None):
         '''
         Get a state file from the master and store it in the local minion
         cache; return the location of the file
@@ -443,21 +472,22 @@ class Client(object):
         sls_url = salt.utils.url.create(sls + '.sls')
         init_url = salt.utils.url.create(sls + '/init.sls')
         for path in [sls_url, init_url]:
-            dest = self.cache_file(path, saltenv)
+            dest = self.cache_file(path, saltenv, cachedir=cachedir)
             if dest:
                 return {'source': path, 'dest': dest}
         return {}
 
-    def get_dir(self, path, dest='', saltenv='base', gzip=None, env=None):
+    def get_dir(self, path, dest='', saltenv='base', gzip=None, env=None,
+                cachedir=None):
         '''
         Get a directory recursively from the salt-master
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -515,16 +545,17 @@ class Client(object):
         ret.sort()
         return ret
 
-    def get_url(self, url, dest, makedirs=False, saltenv='base', env=None, no_cache=False):
+    def get_url(self, url, dest, makedirs=False, saltenv='base', env=None,
+                no_cache=False, cachedir=None):
         '''
         Get a single file from a URL.
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -535,12 +566,13 @@ class Client(object):
             # Local filesystem
             if not os.path.isabs(url_data.path):
                 raise CommandExecutionError(
-                    'Path {0!r} is not absolute'.format(url_data.path)
+                    'Path \'{0}\' is not absolute'.format(url_data.path)
                 )
             return url_data.path
 
         if url_data.scheme == 'salt':
-            return self.get_file(url, dest, makedirs, saltenv)
+            return self.get_file(
+                url, dest, makedirs, saltenv, cachedir=cachedir)
         if dest:
             destdir = os.path.dirname(dest)
             if not os.path.isdir(destdir):
@@ -549,7 +581,7 @@ class Client(object):
                 else:
                     return ''
         elif not no_cache:
-            dest = self._extrn_path(url, saltenv)
+            dest = self._extrn_path(url, saltenv, cachedir=cachedir)
             destdir = os.path.dirname(dest)
             if not os.path.isdir(destdir):
                 os.makedirs(destdir)
@@ -577,7 +609,9 @@ class Client(object):
                                     location=s3_opt('location'))
                 return dest
             except Exception as exc:
-                raise MinionError('Could not fetch from {0}. Exception: {1}'.format(url, exc))
+                raise MinionError(
+                    'Could not fetch from {0}. Exception: {1}'.format(url, exc)
+                )
         if url_data.scheme == 'ftp':
             try:
                 ftp = ftplib.FTP(url_data.hostname)
@@ -590,10 +624,20 @@ class Client(object):
 
         if url_data.scheme == 'swift':
             try:
-                swift_conn = SaltSwift(self.opts.get('keystone.user', None),
-                                       self.opts.get('keystone.tenant', None),
-                                       self.opts.get('keystone.auth_url', None),
-                                       self.opts.get('keystone.password', None))
+                def swift_opt(key, default):
+                    '''Get value of <key> from Minion config or from Pillar'''
+                    if key in self.opts:
+                        return self.opts[key]
+                    try:
+                        return self.opts['pillar'][key]
+                    except (KeyError, TypeError):
+                        return default
+
+                swift_conn = SaltSwift(swift_opt('keystone.user', None),
+                                       swift_opt('keystone.tenant', None),
+                                       swift_opt('keystone.auth_url', None),
+                                       swift_opt('keystone.password', None))
+
                 swift_conn.get_object(url_data.netloc,
                                       url_data.path[1:],
                                       dest)
@@ -604,7 +648,10 @@ class Client(object):
         get_kwargs = {}
         if url_data.username is not None \
                 and url_data.scheme in ('http', 'https'):
-            _, netloc = url_data.netloc.split('@', 1)
+            netloc = url_data.netloc
+            at_sign_pos = netloc.rfind('@')
+            if at_sign_pos != -1:
+                netloc = netloc[at_sign_pos + 1:]
             fixed_url = urlunparse(
                 (url_data.scheme, netloc, url_data.path,
                  url_data.params, url_data.query, url_data.fragment))
@@ -614,21 +661,59 @@ class Client(object):
 
         destfp = None
         try:
+            # Tornado calls streaming_callback on redirect response bodies.
+            # But we need streaming to support fetching large files (> RAM avail).
+            # Here we working this around by disabling recording the body for redirections.
+            # The issue is fixed in Tornado 4.3.0 so on_header callback could be removed
+            # when we'll deprecate Tornado<4.3.0.
+            # See #27093 and #30431 for details.
+
+            # Use list here to make it writable inside the on_header callback. Simple bool doesn't
+            # work here: on_header creates a new local variable instead. This could be avoided in
+            # Py3 with 'nonlocal' statement. There is no Py2 alternative for this.
+            write_body = [False]
+
+            def on_header(hdr):
+                try:
+                    hdr = parse_response_start_line(hdr)
+                except HTTPInputError:
+                    # Not the first line, do nothing
+                    return
+                write_body[0] = hdr.code not in [301, 302, 303, 307]
+
+            if no_cache:
+                result = []
+
+                def on_chunk(chunk):
+                    if write_body[0]:
+                        result.append(chunk)
+            else:
+                dest_tmp = "{0}.part".format(dest)
+                # We need an open filehandle to use in the on_chunk callback,
+                # that's why we're not using a with clause here.
+                destfp = salt.utils.fopen(dest_tmp, 'wb')
+
+                def on_chunk(chunk):
+                    if write_body[0]:
+                        destfp.write(chunk)
+
             query = salt.utils.http.query(
                 fixed_url,
                 stream=True,
+                streaming_callback=on_chunk,
+                header_callback=on_header,
                 username=url_data.username,
                 password=url_data.password,
+                opts=self.opts,
                 **get_kwargs
             )
             if 'handle' not in query:
-                raise MinionError('Error: {0}'.format(query['error']))
+                raise MinionError('Error: {0} reading {1}'.format(query['error'], url))
             if no_cache:
-                return query['body']
+                return ''.join(result)
             else:
-                dest_tmp = "{0}.part".format(dest)
-                with salt.utils.fopen(dest_tmp, 'wb') as destfp:
-                    destfp.write(query['body'])
+                destfp.close()
+                destfp = None
                 salt.utils.files.rename(dest_tmp, dest)
                 return dest
         except HTTPError as exc:
@@ -650,23 +735,24 @@ class Client(object):
             makedirs=False,
             saltenv='base',
             env=None,
+            cachedir=None,
             **kwargs):
         '''
         Cache a file then process it as a template
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
 
         kwargs['saltenv'] = saltenv
         url_data = urlparse(url)
-        sfn = self.cache_file(url, saltenv)
+        sfn = self.cache_file(url, saltenv, cachedir=cachedir)
         if not os.path.exists(sfn):
             return ''
         if template in salt.utils.templates.TEMPLATE_REGISTRY:
@@ -688,7 +774,7 @@ class Client(object):
             return ''
         if not dest:
             # No destination passed, set the dest as an extrn_files cache
-            dest = self._extrn_path(url, saltenv)
+            dest = self._extrn_path(url, saltenv, cachedir=cachedir)
             # If Salt generated the dest name, create any required dirs
             makedirs = True
 
@@ -702,7 +788,7 @@ class Client(object):
         shutil.move(data['data'], dest)
         return dest
 
-    def _extrn_path(self, url, saltenv):
+    def _extrn_path(self, url, saltenv, cachedir=None):
         '''
         Return the extn_filepath for a given url
         '''
@@ -712,8 +798,13 @@ class Client(object):
         else:
             netloc = url_data.netloc
 
+        if cachedir is None:
+            cachedir = self.opts['cachedir']
+        elif not os.path.isabs(cachedir):
+            cachedir = os.path.join(self.opts['cachedir'], cachedir)
+
         return salt.utils.path_join(
-            self.opts['cachedir'],
+            cachedir,
             'extrn_files',
             saltenv,
             netloc,
@@ -754,17 +845,18 @@ class LocalClient(Client):
                  makedirs=False,
                  saltenv='base',
                  gzip=None,
-                 env=None):
+                 env=None,
+                 cachedir=None):
         '''
         Copies a file from the local files directory into :param:`dest`
         gzip compression settings are ignored for local files
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -782,10 +874,10 @@ class LocalClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -810,10 +902,10 @@ class LocalClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -837,10 +929,10 @@ class LocalClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -864,10 +956,10 @@ class LocalClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -901,10 +993,10 @@ class LocalClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -967,14 +1059,14 @@ class RemoteClient(Client):
                  makedirs=False,
                  saltenv='base',
                  gzip=None,
-                 env=None):
+                 env=None,
+                 cachedir=None):
         '''
         Get a single file from the salt-master
         path must be a salt server location, aka, salt://path/to/file, if
         dest is omitted, then the downloaded file will be placed in the minion
         cache
         '''
-
         path, senv = salt.utils.url.split_env(path)
         if senv:
             saltenv = senv
@@ -984,7 +1076,7 @@ class RemoteClient(Client):
         hash_server = self.hash_file(path, saltenv)
         if hash_server == '':
             log.debug(
-                'Could not find file from saltenv {0!r}, {1!r}'.format(
+                'Could not find file from saltenv \'{0}\', \'{1}\''.format(
                     saltenv, path
                 )
             )
@@ -992,10 +1084,10 @@ class RemoteClient(Client):
 
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -1007,34 +1099,32 @@ class RemoteClient(Client):
             rel_path = self._check_proto(path)
 
             log.debug(
-                'In saltenv {0!r}, looking at rel_path {1!r} to resolve {2!r}'.format(
-                    saltenv, rel_path, path
-                )
+                'In saltenv \'{0}\', looking at rel_path \'{1}\' to resolve '
+                '\'{2}\''.format(saltenv, rel_path, path)
             )
-            with self._cache_loc(rel_path, saltenv) as cache_dest:
+            with self._cache_loc(
+                    rel_path, saltenv, cachedir=cachedir) as cache_dest:
                 dest2check = cache_dest
 
         log.debug(
-            'In saltenv {0!r}, ** considering ** path {1!r} to resolve {2!r}'.format(
-                saltenv, dest2check, path
-            )
+            'In saltenv \'{0}\', ** considering ** path \'{1}\' to resolve '
+            '\'{2}\''.format(saltenv, dest2check, path)
         )
 
         if dest2check and os.path.isfile(dest2check):
             hash_local = self.hash_file(dest2check, saltenv)
             if hash_local == hash_server:
                 log.info(
-                    'Fetching file from saltenv {0!r}, ** skipped ** '
-                    'latest already in cache {1!r}'.format(
+                    'Fetching file from saltenv \'{0}\', ** skipped ** '
+                    'latest already in cache \'{1}\''.format(
                         saltenv, path
                     )
                 )
                 return dest2check
 
         log.debug(
-            'Fetching file from saltenv {0!r}, ** attempting ** {1!r}'.format(
-                saltenv, path
-            )
+            'Fetching file from saltenv \'{0}\', ** attempting ** '
+            '\'{1}\''.format(saltenv, path)
         )
         d_tries = 0
         transport_tries = 0
@@ -1054,6 +1144,8 @@ class RemoteClient(Client):
                     os.makedirs(destdir)
                 else:
                     return False
+            # We need an open filehandle here, that's why we're not using a
+            # with clause:
             fn_ = salt.utils.fopen(dest, 'wb+')
         else:
             log.debug('No dest file found {0}'.format(dest))
@@ -1068,7 +1160,10 @@ class RemoteClient(Client):
                 if not data['data']:
                     if not fn_ and data['dest']:
                         # This is a 0 byte file on the master
-                        with self._cache_loc(data['dest'], saltenv) as cache_dest:
+                        with self._cache_loc(
+                                data['dest'],
+                                saltenv,
+                                cachedir=cachedir) as cache_dest:
                             dest = cache_dest
                             with salt.utils.fopen(cache_dest, 'wb+') as ofile:
                                 ofile.write(data['data'])
@@ -1083,7 +1178,10 @@ class RemoteClient(Client):
                             continue
                     break
                 if not fn_:
-                    with self._cache_loc(data['dest'], saltenv) as cache_dest:
+                    with self._cache_loc(
+                            data['dest'],
+                            saltenv,
+                            cachedir=cachedir) as cache_dest:
                         dest = cache_dest
                         # If a directory was formerly cached at this path, then
                         # remove it to avoid a traceback trying to write the file
@@ -1097,26 +1195,29 @@ class RemoteClient(Client):
                 fn_.write(data)
             except (TypeError, KeyError) as e:
                 transport_tries += 1
-                log.error('Data transport is broken, got: {0}, type: {1}, '
-                          'exception: {2}, attempt {3} of 3'.format(
-                              data, type(data), e, transport_tries)
-                          )
+                log.warning('Data transport is broken, got: {0}, type: {1}, '
+                            'exception: {2}, attempt {3} of 3'.format(
+                                data, type(data), e, transport_tries)
+                            )
                 self._refresh_channel()
                 if transport_tries > 3:
+                    log.error('Data transport is broken, got: {0}, type: {1}, '
+                              'exception: {2}, '
+                              'Retry attempts exhausted'.format(
+                                data, type(data), e)
+                            )
                     break
 
         if fn_:
             fn_.close()
             log.info(
-                'Fetching file from saltenv {0!r}, ** done ** {1!r}'.format(
-                    saltenv, path
-                )
+                'Fetching file from saltenv \'{0}\', ** done ** '
+                '\'{1}\''.format(saltenv, path)
             )
         else:
             log.debug(
-                'In saltenv {0!r}, we are ** missing ** the file {1!r}'.format(
-                    saltenv, path
-                )
+                'In saltenv \'{0}\', we are ** missing ** the file '
+                '\'{1}\''.format(saltenv, path)
             )
 
         return dest
@@ -1127,10 +1228,10 @@ class RemoteClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -1139,7 +1240,7 @@ class RemoteClient(Client):
                 'prefix': prefix,
                 'cmd': '_file_list'}
 
-        return self.channel.send(load)
+        return [sdecode(fn_) for fn_ in self.channel.send(load)]
 
     def file_list_emptydirs(self, saltenv='base', prefix='', env=None):
         '''
@@ -1147,10 +1248,10 @@ class RemoteClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -1166,10 +1267,10 @@ class RemoteClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -1196,10 +1297,10 @@ class RemoteClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
@@ -1229,10 +1330,10 @@ class RemoteClient(Client):
         '''
         if env is not None:
             salt.utils.warn_until(
-                'Boron',
+                'Carbon',
                 'Passing a salt environment should be done using \'saltenv\' '
                 'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
+                'Carbon.'
             )
             # Backwards compatibility
             saltenv = env
