@@ -5,7 +5,7 @@ Connection module for Amazon Elasticache
 .. versionadded:: 2014.7.0
 
 :configuration: This module accepts explicit elasticache credentials but can
-    also utilize IAM roles assigned to the instance trough Instance Profiles.
+    also utilize IAM roles assigned to the instance through Instance Profiles.
     Dynamic credentials are then automatically obtained from AWS API and no
     further configuration is necessary. More Information available at:
 
@@ -73,7 +73,7 @@ def __virtual__():
     Only load if boto libraries exist.
     '''
     if not HAS_BOTO:
-        return False
+        return (False, 'The modle boto_elasticache could not be loaded: boto libraries not found')
     __utils__['boto.assign_funcs'](__name__, 'elasticache', pack=__salt__)
     return True
 
