@@ -2,6 +2,12 @@
 '''
 A module to wrap pacman calls, since Arch is the best
 (https://wiki.archlinux.org/index.php/Arch_is_the_best)
+
+.. important::
+    If you feel that Salt should be using this module to manage packages on a
+    minion, and it is using a different module (or gives an error similar to
+    *'pkg.install' is not available*), see :ref:`here
+    <module-provider-override>`.
 '''
 
 # Import python libs
@@ -9,9 +15,12 @@ from __future__ import absolute_import
 import copy
 import logging
 import re
+import os.path
 
 # Import salt libs
 import salt.utils
+import salt.utils.itertools
+import salt.utils.systemd
 from salt.exceptions import CommandExecutionError, MinionError
 
 # Import 3rd-party libs
@@ -27,9 +36,9 @@ def __virtual__():
     '''
     Set the virtual pkg module if the os is Arch
     '''
-    if __grains__['os'] in ('Arch', 'Arch ARM'):
+    if __grains__['os'] in ('Arch', 'Arch ARM', 'Antergos', 'ManjaroLinux'):
         return __virtualname__
-    return False
+    return (False, 'The pacman module could not be loaded: unsupported OS family.')
 
 
 def _list_removed(old, new):
@@ -68,10 +77,16 @@ def latest_version(*names, **kwargs):
     # Initialize the dict with empty strings
     for name in names:
         ret[name] = ''
-    cmd = 'pacman -Sp --needed --print-format "%n %v" ' \
-          '{0}'.format(' '.join(names))
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Sp', '--needed', '--print-format', '%n %v']
+    cmd.extend(names)
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
+    out = __salt__['cmd.run_stdout'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         try:
             name, version_num = line.split()
             # Only add to return dict if package is in the list of packages
@@ -112,7 +127,7 @@ def upgrade_available(name):
     return latest_version(name) != ''
 
 
-def list_upgrades(refresh=False):
+def list_upgrades(refresh=False, root=None, **kwargs):  # pylint: disable=W0613
     '''
     List all available package upgrades on this system
 
@@ -123,14 +138,17 @@ def list_upgrades(refresh=False):
         salt '*' pkg.list_upgrades
     '''
     upgrades = {}
-    options = ['-S', '-p', '-u', '--print-format "%n %v"']
+    cmd = ['pacman', '-S', '-p', '-u', '--print-format', '%n %v']
+
+    if root is not None:
+        cmd.extend(('-r', root))
 
     if refresh:
-        options.append('-y')
+        cmd.append('-y')
 
-    cmd = ('pacman {0}').format(' '.join(options))
-
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
+    call = __salt__['cmd.run_all'](cmd,
+                                   python_shell=False,
+                                   output_loglevel='trace')
 
     if call['retcode'] != 0:
         comment = ''
@@ -138,13 +156,13 @@ def list_upgrades(refresh=False):
             comment += call['stderr']
         if 'stdout' in call:
             comment += call['stdout']
-        raise CommandExecutionError(
-            '{0}'.format(comment)
-        )
+        if comment:
+            comment = ': ' + comment
+        raise CommandExecutionError('Error listing upgrades' + comment)
     else:
         out = call['stdout']
 
-    for line in iter(out.splitlines()):
+    for line in salt.utils.itertools.split(out, '\n'):
         comps = line.split(' ')
         if len(comps) != 2:
             continue
@@ -194,17 +212,21 @@ def list_pkgs(versions_as_list=False, **kwargs):
             __salt__['pkg_resource.stringify'](ret)
             return ret
 
-    cmd = 'pacman -Q'
+    cmd = ['pacman', '-Q']
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
     ret = {}
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if not line:
             continue
         try:
             name, version_num = line.split()[0:2]
         except ValueError:
             log.error('Problem parsing pacman -Q: Unexpected formatting in '
-                      'line: "{0}"'.format(line))
+                      'line: \'{0}\''.format(line))
         else:
             __salt__['pkg_resource.add_pkg'](ret, name, version_num)
 
@@ -215,7 +237,7 @@ def list_pkgs(versions_as_list=False, **kwargs):
     return ret
 
 
-def refresh_db():
+def refresh_db(root=None):
     '''
     Just run a ``pacman -Sy``, return a dict::
 
@@ -227,22 +249,27 @@ def refresh_db():
 
         salt '*' pkg.refresh_db
     '''
-    cmd = 'LANG=C pacman -Sy'
+    cmd = ['pacman', '-Sy']
+
+    if root is not None:
+        cmd.extend(('-r', root))
+
     ret = {}
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace',
-            python_shell=True)
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   env={'LANG': 'C'},
+                                   python_shell=False)
     if call['retcode'] != 0:
         comment = ''
         if 'stderr' in call:
-            comment += call['stderr']
-
+            comment += ': ' + call['stderr']
         raise CommandExecutionError(
-            '{0}'.format(comment)
+            'Error refreshing package database' + comment
         )
     else:
         out = call['stdout']
 
-    for line in out.splitlines():
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.strip().startswith('::'):
             continue
         if not line:
@@ -263,15 +290,29 @@ def install(name=None,
             sources=None,
             **kwargs):
     '''
-    Install (pacman -S) the passed package, add refresh=True to install with
-    -y, add sysupgrade=True to install with -u.
+    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+        On minions running systemd>=205, `systemd-run(1)`_ is now used to
+        isolate commands which modify installed packages from the
+        ``salt-minion`` daemon's control group. This is done to keep systemd
+        from killing any pacman commands spawned by Salt when the
+        ``salt-minion`` service is restarted. (see ``KillMode`` in the
+        `systemd.kill(5)`_ manpage for more information). If desired, usage of
+        `systemd-run(1)`_ can be suppressed by setting a :mod:`config option
+        <salt.modules.config.get>` called ``systemd.scope``, with a value of
+        ``False`` (no quotes).
+
+    .. _`systemd-run(1)`: https://www.freedesktop.org/software/systemd/man/systemd-run.html
+    .. _`systemd.kill(5)`: https://www.freedesktop.org/software/systemd/man/systemd.kill.html
+
+    Install (``pacman -S``) the specified packag(s). Add ``refresh=True`` to
+    install with ``-y``, add ``sysupgrade=True`` to install with ``-u``.
 
     name
         The name of the package to be installed. Note that this parameter is
-        ignored if either "pkgs" or "sources" is passed. Additionally, please
-        note that this option can only be used to install packages from a
-        software repository. To install a package file manually, use the
-        "sources" option.
+        ignored if either ``pkgs`` or ``sources`` is passed. Additionally,
+        please note that this option can only be used to install packages from
+        a software repository. To install a package file manually, use the
+        ``sources`` option.
 
         CLI Example:
 
@@ -335,20 +376,33 @@ def install(name=None,
     version_num = kwargs.get('version')
     if version_num:
         if pkgs is None and sources is None:
-            # Allow "version" to work for single package target
+            # Allow 'version' to work for single package target
             pkg_params = {name: version_num}
         else:
-            log.warning('"version" parameter will be ignored for multiple '
+            log.warning('\'version\' parameter will be ignored for multiple '
                         'package targets')
 
+    if 'root' in kwargs:
+        pkg_params['-r'] = kwargs['root']
+
+    cmd = []
+    if salt.utils.systemd.has_scope(__context__) \
+            and __salt__['config.get']('systemd.scope', True):
+        cmd.extend(['systemd-run', '--scope'])
+    cmd.append('pacman')
+
     if pkg_type == 'file':
-        cmd = 'pacman -U --noprogressbar --noconfirm ' \
-              '{0}'.format(' '.join(pkg_params))
-        targets = pkg_params
+        cmd.extend(['-U', '--noprogressbar', '--noconfirm'])
+        cmd.extend(pkg_params)
     elif pkg_type == 'repository':
+        cmd.append('-S')
+        if salt.utils.is_true(refresh):
+            cmd.append('-y')
+        if salt.utils.is_true(sysupgrade):
+            cmd.append('-u')
+        cmd.extend(['--noprogressbar', '--noconfirm', '--needed'])
         targets = []
         problems = []
-        options = ['--noprogressbar', '--noconfirm', '--needed']
         for param, version_num in six.iteritems(pkg_params):
             if version_num is None:
                 targets.append(param)
@@ -362,30 +416,57 @@ def install(name=None,
                     prefix = prefix or '='
                     targets.append('{0}{1}{2}'.format(param, prefix, verstr))
                 else:
-                    msg = 'Invalid version string "{0}" for package ' \
-                          '"{1}"'.format(version_num, name)
+                    msg = ('Invalid version string \'{0}\' for package '
+                           '\'{1}\''.format(version_num, name))
                     problems.append(msg)
         if problems:
             for problem in problems:
                 log.error(problem)
             return {}
 
-        if salt.utils.is_true(refresh):
-            options.append('-y')
-        if salt.utils.is_true(sysupgrade):
-            options.append('-u')
-
-        cmd = 'pacman -S "{0}"'.format('" "'.join(options+targets))
+        cmd.extend(targets)
 
     old = list_pkgs()
-    __salt__['cmd.run'](cmd, output_loglevel='trace')
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
-def upgrade(refresh=False, **kwargs):
+def upgrade(refresh=False, root=None, **kwargs):
     '''
+    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+        On minions running systemd>=205, `systemd-run(1)`_ is now used to
+        isolate commands which modify installed packages from the
+        ``salt-minion`` daemon's control group. This is done to keep systemd
+        from killing any pacman commands spawned by Salt when the
+        ``salt-minion`` service is restarted. (see ``KillMode`` in the
+        `systemd.kill(5)`_ manpage for more information). If desired, usage of
+        `systemd-run(1)`_ can be suppressed by setting a :mod:`config option
+        <salt.modules.config.get>` called ``systemd.scope``, with a value of
+        ``False`` (no quotes).
+
+    .. _`systemd-run(1)`: https://www.freedesktop.org/software/systemd/man/systemd-run.html
+    .. _`systemd.kill(5)`: https://www.freedesktop.org/software/systemd/man/systemd.kill.html
+
     Run a full system upgrade, a pacman -Syu
 
     refresh
@@ -404,24 +485,35 @@ def upgrade(refresh=False, **kwargs):
     '''
     ret = {'changes': {},
            'result': True,
-           'comment': '',
-           }
+           'comment': ''}
 
     old = list_pkgs()
-    cmd = 'pacman -Su --noprogressbar --noconfirm'
+
+    cmd = []
+    if salt.utils.systemd.has_scope(__context__) \
+            and __salt__['config.get']('systemd.scope', True):
+        cmd.extend(['systemd-run', '--scope'])
+    cmd.extend(['pacman', '-Su', '--noprogressbar', '--noconfirm'])
     if salt.utils.is_true(refresh):
-        cmd += ' -y'
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
+        cmd.append('-y')
+
+    if root is not None:
+        cmd.extend(('-r', root))
+
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   python_shell=False,
+                                   redirect_stderr=True)
+
     if call['retcode'] != 0:
         ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+        if call['stdout']:
+            ret['comment'] = call['stdout']
+
+    __context__.pop('pkg.list_pkgs', None)
+    new = list_pkgs()
+    ret['changes'] = salt.utils.compare_dicts(old, new)
+
     return ret
 
 
@@ -439,20 +531,59 @@ def _uninstall(action='remove', name=None, pkgs=None, **kwargs):
     targets = [x for x in pkg_params if x in old]
     if not targets:
         return {}
+
     remove_arg = '-Rsc' if action == 'purge' else '-R'
-    cmd = (
-        'pacman {0} '
-        '--noprogressbar '
-        '--noconfirm {1}'
-    ).format(remove_arg, ' '.join(targets))
-    __salt__['cmd.run'](cmd, output_loglevel='trace')
+
+    cmd = []
+    if salt.utils.systemd.has_scope(__context__) \
+            and __salt__['config.get']('systemd.scope', True):
+        cmd.extend(['systemd-run', '--scope'])
+    cmd.extend(['pacman', remove_arg, '--noprogressbar', '--noconfirm'])
+    cmd.extend(targets)
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None, pkgs=None, **kwargs):
     '''
+    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+        On minions running systemd>=205, `systemd-run(1)`_ is now used to
+        isolate commands which modify installed packages from the
+        ``salt-minion`` daemon's control group. This is done to keep systemd
+        from killing any pacman commands spawned by Salt when the
+        ``salt-minion`` service is restarted. (see ``KillMode`` in the
+        `systemd.kill(5)`_ manpage for more information). If desired, usage of
+        `systemd-run(1)`_ can be suppressed by setting a :mod:`config option
+        <salt.modules.config.get>` called ``systemd.scope``, with a value of
+        ``False`` (no quotes).
+
+    .. _`systemd-run(1)`: https://www.freedesktop.org/software/systemd/man/systemd-run.html
+    .. _`systemd.kill(5)`: https://www.freedesktop.org/software/systemd/man/systemd.kill.html
+
     Remove packages with ``pacman -R``.
 
     name
@@ -483,6 +614,20 @@ def remove(name=None, pkgs=None, **kwargs):
 
 def purge(name=None, pkgs=None, **kwargs):
     '''
+    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+        On minions running systemd>=205, `systemd-run(1)`_ is now used to
+        isolate commands which modify installed packages from the
+        ``salt-minion`` daemon's control group. This is done to keep systemd
+        from killing any pacman commands spawned by Salt when the
+        ``salt-minion`` service is restarted. (see ``KillMode`` in the
+        `systemd.kill(5)`_ manpage for more information). If desired, usage of
+        `systemd-run(1)`_ can be suppressed by setting a :mod:`config option
+        <salt.modules.config.get>` called ``systemd.scope``, with a value of
+        ``False`` (no quotes).
+
+    .. _`systemd-run(1)`: https://www.freedesktop.org/software/systemd/man/systemd-run.html
+    .. _`systemd.kill(5)`: https://www.freedesktop.org/software/systemd/man/systemd.kill.html
+
     Recursively remove a package and all dependencies which were installed
     with it, this will call a ``pacman -Rsc``
 
@@ -528,9 +673,16 @@ def file_list(*packages):
     '''
     errors = []
     ret = []
-    cmd = 'pacman -Ql {0}'.format(' '.join(packages))
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Ql']
+
+    if len(packages) > 0 and os.path.exists(packages[0]):
+        packages = list(packages)
+        cmd.extend(('-r', packages.pop(0)))
+
+    cmd.extend(packages)
+
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.startswith('error'):
             errors.append(line)
         else:
@@ -555,9 +707,16 @@ def file_dict(*packages):
     '''
     errors = []
     ret = {}
-    cmd = 'pacman -Ql {0}'.format(' '.join(packages))
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Ql']
+
+    if len(packages) > 0 and os.path.exists(packages[0]):
+        packages = list(packages)
+        cmd.extend(('-r', packages.pop(0)))
+
+    cmd.extend(packages)
+
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.startswith('error'):
             errors.append(line)
         else:
@@ -588,9 +747,11 @@ def owner(*paths):
     if not paths:
         return ''
     ret = {}
-    cmd = 'pacman -Qqo {0!r}'
+    cmd_prefix = ['pacman', '-Qqo']
+
     for path in paths:
-        ret[path] = __salt__['cmd.run_stdout'](cmd.format(path))
+        ret[path] = __salt__['cmd.run_stdout'](cmd_prefix + [path],
+                                               python_shell=False)
     if len(ret) == 1:
         return next(six.itervalues(ret))
     return ret
