@@ -101,8 +101,6 @@ class NetapiClient(object):
         '''
         Run :ref:`execution modules <all-salt.modules>` against batches of minions
 
-        .. versionadded:: 0.8.4
-
         Wraps :py:meth:`salt.client.LocalClient.cmd_batch`
 
         :return: Returns the result from the exeuction module for each batch of
@@ -111,6 +109,17 @@ class NetapiClient(object):
         local = salt.client.get_local_client(mopts=self.opts)
         return local.cmd_batch(*args, **kwargs)
 
+    def local_subset(self, *args, **kwargs):
+        '''
+        Run :ref:`execution modules <all-salt.modules>` against subsets of minions
+
+        .. versionadded:: 2016.3.0
+
+        Wraps :py:meth:`salt.client.LocalClient.cmd_subset`
+        '''
+        local = salt.client.get_local_client(mopts=self.opts)
+        return local.cmd_subset(*args, **kwargs)
+
     def ssh(self, *args, **kwargs):
         '''
         Run salt-ssh commands synchronously
